@@ -21,6 +21,7 @@
 #include <boost/tuple/tuple.hpp>
 
 #include "app_state.hh"
+#include "project.hh"
 #include "basic_io.hh"
 #include "cert.hh"
 #include "cmd.hh"
@@ -28,15 +29,21 @@
 #include "constants.hh"
 #include "inodeprint.hh"
 #include "keys.hh"
+#include "key_store.hh"
 #include "file_io.hh"
 #include "packet.hh"
 #include "restrictions.hh"
 #include "revision.hh"
+#include "roster.hh"
 #include "transforms.hh"
+#include "simplestring_xform.hh"
 #include "vocab.hh"
 #include "globish.hh"
 #include "charset.hh"
 #include "safe_map.hh"
+#include "work.hh"
+#include "xdelta.hh"
+#include "database.hh"
 
 using std::allocator;
 using std::basic_ios;
@@ -73,21 +80,24 @@ CMD_AUTOMATE(heads, N_("[BRANCH]"),
   N(args.size() < 2,
     F("wrong argument count"));
 
-  system_path database_option;
-  branch_name branch_option;
-  rsa_keypair_id key_option;
-  system_path keydir_option;
-  app.work.get_ws_options(database_option, branch_option,
-                          key_option, keydir_option);
+  database db(app);
+  project_t project(db);
 
-  if (args.size() == 1 ) {
+  branch_name branch;
+  if (args.size() == 1)
     // branchname was explicitly given, use that
-    branch_option = branch_name(idx(args, 0)());
-  }
+    branch = branch_name(idx(args, 0)());
+  else
+    {
+      workspace::require_workspace(F("with no argument, this command prints the heads of the workspace's branch"));
+      branch = app.opts.branchname;
+    }
+
   set<revision_id> heads;
-  app.get_project().get_branch_heads(branch_option, heads);
-  for (set<revision_id>::const_iterator i = heads.begin(); i != heads.end(); ++i)
-    output << (*i).inner()() << '\n';
+  project.get_branch_heads(branch, heads, app.opts.ignore_suspend_certs);
+  for (set<revision_id>::const_iterator i = heads.begin();
+       i != heads.end(); ++i)
+    output << *i << '\n';
 }
 
 // Name: ancestors
@@ -107,12 +117,14 @@ CMD_AUTOMATE(ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
   N(args.size() > 0,
     F("wrong argument count"));
 
+  database db(app);
+
   set<revision_id> ancestors;
   vector<revision_id> frontier;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid((*i)());
-      N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+      revision_id rid(decode_hexenc((*i)()));
+      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
       frontier.push_back(rid);
     }
   while (!frontier.empty())
@@ -121,7 +133,7 @@ CMD_AUTOMATE(ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
       frontier.pop_back();
       if(!null_id(rid)) {
         set<revision_id> parents;
-        app.db.get_revision_parents(rid, parents);
+        db.get_revision_parents(rid, parents);
         for (set<revision_id>::const_iterator i = parents.begin();
              i != parents.end(); ++i)
           {
@@ -136,7 +148,7 @@ CMD_AUTOMATE(ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
   for (set<revision_id>::const_iterator i = ancestors.begin();
        i != ancestors.end(); ++i)
     if (!null_id(*i))
-      output << (*i).inner()() << '\n';
+      output << *i << '\n';
 }
 
 
@@ -157,12 +169,14 @@ CMD_AUTOMATE(descendents, N_("REV1 [REV2 [REV3 [...]]]"),
   N(args.size() > 0,
     F("wrong argument count"));
 
+  database db(app);
+
   set<revision_id> descendents;
   vector<revision_id> frontier;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid((*i)());
-      N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+      revision_id rid(decode_hexenc((*i)()));
+      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
       frontier.push_back(rid);
     }
   while (!frontier.empty())
@@ -170,7 +184,7 @@ CMD_AUTOMATE(descendents, N_("REV1 [REV2 [REV3 [...]]]"),
       revision_id rid = frontier.back();
       frontier.pop_back();
       set<revision_id> children;
-      app.db.get_revision_children(rid, children);
+      db.get_revision_children(rid, children);
       for (set<revision_id>::const_iterator i = children.begin();
            i != children.end(); ++i)
         {
@@ -183,7 +197,7 @@ CMD_AUTOMATE(descendents, N_("REV1 [REV2 [REV3 [...]]]"),
     }
   for (set<revision_id>::const_iterator i = descendents.begin();
        i != descendents.end(); ++i)
-    output << (*i).inner()() << '\n';
+    output << *i << '\n';
 }
 
 
@@ -205,16 +219,18 @@ CMD_AUTOMATE(erase_ancestors, N_("[REV1 [REV2 [REV3 [...]]]]"),
              "",
              options::opts::none)
 {
+  database db(app);
+
   set<revision_id> revs;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid((*i)());
-      N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+      revision_id rid(decode_hexenc((*i)()));
+      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
       revs.insert(rid);
     }
-  erase_ancestors(revs, app);
+  erase_ancestors(db, revs);
   for (set<revision_id>::const_iterator i = revs.begin(); i != revs.end(); ++i)
-    output << (*i).inner()() << '\n';
+    output << *i << '\n';
 }
 
 // Name: toposort
@@ -232,18 +248,20 @@ CMD_AUTOMATE(toposort, N_("[REV1 [REV2 [REV3 [...]]]]"),
              "",
              options::opts::none)
 {
+  database db(app);
+
   set<revision_id> revs;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid((*i)());
-      N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+      revision_id rid(decode_hexenc((*i)()));
+      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
       revs.insert(rid);
     }
   vector<revision_id> sorted;
-  toposort(revs, sorted, app);
+  toposort(db, revs, sorted);
   for (vector<revision_id>::const_iterator i = sorted.begin();
        i != sorted.end(); ++i)
-    output << (*i).inner()() << '\n';
+    output << *i << '\n';
 }
 
 // Name: ancestry_difference
@@ -271,25 +289,27 @@ CMD_AUTOMATE(ancestry_difference, N_("NEW_REV [OLD_REV1 [OLD_REV2 [...]]]"),
   N(args.size() > 0,
     F("wrong argument count"));
 
+  database db(app);
+
   revision_id a;
   set<revision_id> bs;
   args_vector::const_iterator i = args.begin();
-  a = revision_id((*i)());
-  N(app.db.revision_exists(a), F("No such revision %s") % a);
+  a = revision_id(decode_hexenc((*i)()));
+  N(db.revision_exists(a), F("no such revision '%s'") % a);
   for (++i; i != args.end(); ++i)
     {
-      revision_id b((*i)());
-      N(app.db.revision_exists(b), F("No such revision %s") % b);
+      revision_id b(decode_hexenc((*i)()));
+      N(db.revision_exists(b), F("no such revision '%s'") % b);
       bs.insert(b);
     }
   set<revision_id> ancestors;
-  ancestry_difference(a, bs, ancestors, app);
+  ancestry_difference(db, a, bs, ancestors);
 
   vector<revision_id> sorted;
-  toposort(ancestors, sorted, app);
+  toposort(db, ancestors, sorted);
   for (vector<revision_id>::const_iterator i = sorted.begin();
        i != sorted.end(); ++i)
-    output << (*i).inner()() << '\n';
+    output << *i << '\n';
 }
 
 // Name: leaves
@@ -313,11 +333,13 @@ CMD_AUTOMATE(leaves, "",
   N(args.size() == 0,
     F("no arguments needed"));
 
+  database db(app);
+
   set<revision_id> leaves;
-  app.db.get_leaves(leaves);
+  db.get_leaves(leaves);
   for (set<revision_id>::const_iterator i = leaves.begin();
        i != leaves.end(); ++i)
-    output << (*i).inner()() << '\n';
+    output << *i << '\n';
 }
 
 // Name: roots
@@ -337,14 +359,16 @@ CMD_AUTOMATE(roots, "",
   N(args.size() == 0,
     F("no arguments needed"));
 
+  database db(app);
+
   // the real root revisions are the children of one single imaginary root
   // with an empty revision id
   set<revision_id> roots;
   revision_id nullid;
-  app.db.get_revision_children(nullid, roots);
+  db.get_revision_children(nullid, roots);
   for (set<revision_id>::const_iterator i = roots.begin();
        i != roots.end(); ++i)
-      output << i->inner()() << '\n';
+      output << *i << '\n';
 }
 
 // Name: parents
@@ -365,14 +389,16 @@ CMD_AUTOMATE(parents, N_("REV"),
   N(args.size() == 1,
     F("wrong argument count"));
 
-  revision_id rid(idx(args, 0)());
-  N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+  database db(app);
+
+  revision_id rid(decode_hexenc(idx(args, 0)()));
+  N(db.revision_exists(rid), F("no such revision '%s'") % rid);
   set<revision_id> parents;
-  app.db.get_revision_parents(rid, parents);
+  db.get_revision_parents(rid, parents);
   for (set<revision_id>::const_iterator i = parents.begin();
        i != parents.end(); ++i)
       if (!null_id(*i))
-          output << (*i).inner()() << '\n';
+          output << *i << '\n';
 }
 
 // Name: children
@@ -393,14 +419,16 @@ CMD_AUTOMATE(children, N_("REV"),
   N(args.size() == 1,
     F("wrong argument count"));
 
-  revision_id rid(idx(args, 0)());
-  N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+  database db(app);
+
+  revision_id rid(decode_hexenc(idx(args, 0)()));
+  N(db.revision_exists(rid), F("no such revision '%s'") % rid);
   set<revision_id> children;
-  app.db.get_revision_children(rid, children);
+  db.get_revision_children(rid, children);
   for (set<revision_id>::const_iterator i = children.begin();
        i != children.end(); ++i)
       if (!null_id(*i))
-          output << (*i).inner()() << '\n';
+          output << *i << '\n';
 }
 
 // Name: graph
@@ -431,10 +459,12 @@ CMD_AUTOMATE(graph, "",
   N(args.size() == 0,
     F("no arguments needed"));
 
+  database db(app);
+
   multimap<revision_id, revision_id> edges_mmap;
   map<revision_id, set<revision_id> > child_to_parents;
 
-  app.db.get_revision_ancestry(edges_mmap);
+  db.get_revision_ancestry(edges_mmap);
 
   for (multimap<revision_id, revision_id>::const_iterator i = edges_mmap.begin();
        i != edges_mmap.end(); ++i)
@@ -453,10 +483,10 @@ CMD_AUTOMATE(graph, "",
          i = child_to_parents.begin();
        i != child_to_parents.end(); ++i)
     {
-      output << (i->first).inner()();
+      output << i->first;
       for (set<revision_id>::const_iterator j = i->second.begin();
            j != i->second.end(); ++j)
-        output << ' ' << (*j).inner()();
+        output << ' ' << *j;
       output << '\n';
     }
 }
@@ -477,15 +507,12 @@ CMD_AUTOMATE(select, N_("SELECTOR"),
   N(args.size() == 1,
     F("wrong argument count"));
 
-  vector<pair<selectors::selector_type, string> >
-    sels(selectors::parse_selector(args[0](), app));
+  database db(app);
+  project_t project(db);
+  set<revision_id> completions;
+  expand_selector(app.opts, app.lua, project, idx(args, 0)(), completions);
 
-  // we jam through an "empty" selection on sel_ident type
-  set<string> completions;
-  selectors::selector_type ty = selectors::sel_ident;
-  selectors::complete_selector("", sels, ty, completions, app);
-
-  for (set<string>::const_iterator i = completions.begin();
+  for (set<revision_id>::const_iterator i = completions.begin();
        i != completions.end(); ++i)
     output << *i << '\n';
 }
@@ -726,16 +753,18 @@ struct inventory_itemizer : public tree_walker
 {
   path_restriction const & mask;
   inventory_map & inventory;
-  app_state & app;
   inodeprint_map ipm;
+  workspace & work;
 
-  inventory_itemizer(path_restriction const & m, inventory_map & i, app_state & a) :
-    mask(m), inventory(i), app(a)
+  inventory_itemizer(workspace & work,
+                     path_restriction const & m,
+                     inventory_map & i)
+    : mask(m), inventory(i), work(work)
   {
-    if (app.work.in_inodeprints_mode())
+    if (work.in_inodeprints_mode())
       {
         data dat;
-        app.work.read_inodeprints(dat);
+        work.read_inodeprints(dat);
         read_inodeprint_map(dat, ipm);
       }
   }
@@ -751,7 +780,7 @@ inventory_itemizer::visit_dir(file_path const & path)
       inventory[path].fs_type = path::directory;
     }
   // don't recurse into ignored subdirectories
-  return !app.lua.hook_ignore_file(path);
+  return !work.ignore_file(path);
 }
 
 void
@@ -774,9 +803,11 @@ inventory_itemizer::visit_file(file_path const & path)
 }
 
 static void
-inventory_filesystem(path_restriction const & mask, inventory_map & inventory, app_state & app)
+inventory_filesystem(workspace & work,
+                     path_restriction const & mask,
+                     inventory_map & inventory)
 {
-  inventory_itemizer itemizer(mask, inventory, app);
+  inventory_itemizer itemizer(work, mask, inventory);
   file_path const root;
   // The constructor file_path() returns ""; the root directory. walk_tree
   // does not visit that node, so set fs_type now, if it meets the
@@ -804,7 +835,7 @@ namespace
 }
 
 static void
-inventory_determine_states(app_state & app, file_path const & fs_path,
+inventory_determine_states(workspace & work, file_path const & fs_path,
                            inventory_item const & item, roster_t const & old_roster,
                            roster_t const & new_roster, vector<string> & states)
 {
@@ -854,7 +885,7 @@ inventory_determine_states(app_state & app, file_path const & fs_path,
     {
       if (!item.new_node.exists)
         {
-          if (app.lua.hook_ignore_file(fs_path))
+          if (work.ignore_file(fs_path))
             {
               states.push_back("ignored");
             }
@@ -952,10 +983,11 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
              options::opts::no_unchanged |
              options::opts::no_corresponding_renames)
 {
-  app.require_workspace();
+  database db(app);
+  workspace work(app);
 
   parent_map parents;
-  app.work.get_parent_rosters(parents);
+  work.get_parent_rosters(db, parents);
   // for now, until we've figured out what the format could look like
   // and what conceptional model we can implement
   // see: http://www.venge.net/mtn-wiki/MultiParentWorkspaceFallout
@@ -965,7 +997,7 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
   roster_t new_roster, old_roster = parent_roster(parents.begin());
   temp_node_id_source nis;
 
-  app.work.get_current_roster_shape(new_roster, nis);
+  work.get_current_roster_shape(db, nis, new_roster);
 
   inventory_map inventory;
   vector<file_path> includes = args_to_paths(args);
@@ -985,14 +1017,14 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
            inserter(excludes, excludes.end()));
     }
 
-  node_restriction nmask(includes, excludes, app.opts.depth, old_roster, new_roster, app);
+  node_restriction nmask(work, includes, excludes, app.opts.depth, old_roster, new_roster);
   // skip the check of the workspace paths because some of them might
   // be missing and the user might want to query the recorded structure
   // of them anyways
-  path_restriction pmask(includes, excludes, app.opts.depth, app, path_restriction::skip_check);
+  path_restriction pmask(work, includes, excludes, app.opts.depth, path_restriction::skip_check);
 
   inventory_rosters(old_roster, new_roster, nmask, pmask, inventory);
-  inventory_filesystem(pmask, inventory, app);
+  inventory_filesystem(work, pmask, inventory);
 
   basic_io::printer pr;
 
@@ -1006,7 +1038,8 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
       // check if we should output this element at all
       //
       vector<string> states;
-      inventory_determine_states(app, fp, item, old_roster, new_roster, states);
+      inventory_determine_states(work, fp, item,
+                                 old_roster, new_roster, states);
 
       if (find(states.begin(), states.end(), "ignored") != states.end() &&
           app.opts.no_ignored)
@@ -1163,16 +1196,16 @@ CMD_AUTOMATE(get_revision, N_("REVID"),
   N(args.size() == 1,
     F("wrong argument count"));
 
+  database db(app);
+
   revision_data dat;
-  revision_id ident;
+  revision_id rid(decode_hexenc(idx(args, 0)()));
+  N(db.revision_exists(rid),
+    F("no revision %s found in database") % rid);
+  db.get_revision(rid, dat);
 
-  ident = revision_id(idx(args, 0)());
-  N(app.db.revision_exists(ident),
-    F("no revision %s found in database") % ident);
-  app.db.get_revision(ident, dat);
-
-  L(FL("dumping revision %s") % ident);
-  output.write(dat.inner()().data(), dat.inner()().size());
+  L(FL("dumping revision %s") % rid);
+  output << dat;
 }
 
 // Name: get_current_revision
@@ -1198,20 +1231,21 @@ CMD_AUTOMATE(get_current_revision, N_("[PATHS ...]"),
   revision_t rev;
   cset excluded;
 
-  app.require_workspace();
-  app.work.get_parent_rosters(old_rosters);
-  app.work.get_current_roster_shape(new_roster, nis);
+  database db(app);
+  workspace work(app);
+  work.get_parent_rosters(db, old_rosters);
+  work.get_current_roster_shape(db, nis, new_roster);
 
   node_restriction mask(args_to_paths(args),
                         args_to_paths(app.opts.exclude_patterns),
                         app.opts.depth,
-                        old_rosters, new_roster, app);
+                        old_rosters, new_roster);
 
-  app.work.update_current_roster_from_filesystem(new_roster, mask);
+  work.update_current_roster_from_filesystem(new_roster, mask);
 
   make_revision(old_rosters, new_roster, rev);
   make_restricted_revision(old_rosters, new_roster, mask, rev,
-                           excluded, execid);
+                           excluded, join_words(execid));
   rev.check_sane();
   N(rev.is_nontrivial(), F("no changes to commit"));
   
@@ -1219,7 +1253,7 @@ CMD_AUTOMATE(get_current_revision, N_("[PATHS ...]"),
   write_revision(rev, dat);
 
   L(FL("dumping revision %s") % ident);
-  output.write(dat.inner()().data(), dat.inner()().size());
+  output << dat;
 }
 
 
@@ -1238,10 +1272,11 @@ CMD_AUTOMATE(get_base_revision_id, "",
   N(args.size() == 0,
     F("no arguments needed"));
 
-  app.require_workspace();
+  database db(app);
+  workspace work(app);
 
   parent_map parents;
-  app.work.get_parent_rosters(parents);
+  work.get_parent_rosters(db, parents);
   N(parents.size() == 1,
     F("this command can only be used in a single-parent workspace"));
 
@@ -1265,7 +1300,8 @@ CMD_AUTOMATE(get_current_revision_id, "",
   N(args.size() == 0,
     F("no arguments needed"));
 
-  app.require_workspace();
+  workspace work(app);
+  database db(app);
 
   parent_map parents;
   roster_t new_roster;
@@ -1273,11 +1309,10 @@ CMD_AUTOMATE(get_current_revision_id, "",
   revision_t rev;
   temp_node_id_source nis;
 
-  app.require_workspace();
-  app.work.get_current_roster_shape(new_roster, nis);
-  app.work.update_current_roster_from_filesystem(new_roster);
+  work.get_current_roster_shape(db, nis, new_roster);
+  work.update_current_roster_from_filesystem(new_roster);
 
-  app.work.get_parent_rosters(parents);
+  work.get_parent_rosters(db, parents);
   make_revision(parents, new_roster, rev);
 
   calculate_ident(rev, new_revision_id);
@@ -1331,6 +1366,8 @@ CMD_AUTOMATE(get_manifest_of, N_("[REVID]"),
              "",
              options::opts::none)
 {
+  database db(app);
+
   N(args.size() < 2,
     F("wrong argument count"));
 
@@ -1340,24 +1377,25 @@ CMD_AUTOMATE(get_manifest_of, N_("[REVID]"),
 
   if (args.size() == 0)
     {
+      workspace work(app);
+
       temp_node_id_source nis;
 
-      app.require_workspace();
-      app.work.get_current_roster_shape(new_roster, nis);
-      app.work.update_current_roster_from_filesystem(new_roster);
+      work.get_current_roster_shape(db, nis, new_roster);
+      work.update_current_roster_from_filesystem(new_roster);
     }
   else
     {
-      revision_id rid = revision_id(idx(args, 0)());
-      N(app.db.revision_exists(rid),
+      revision_id rid = revision_id(decode_hexenc(idx(args, 0)()));
+      N(db.revision_exists(rid),
         F("no revision %s found in database") % rid);
-      app.db.get_roster(rid, new_roster);
+      db.get_roster(rid, new_roster);
     }
 
   calculate_ident(new_roster, mid);
   write_manifest_of_roster(new_roster, dat);
   L(FL("dumping manifest %s") % mid);
-  output.write(dat.inner()().data(), dat.inner()().size());
+  output << dat;
 }
 
 
@@ -1380,15 +1418,16 @@ CMD_AUTOMATE(packet_for_rdata, N_("REVID"),
   N(args.size() == 1,
     F("wrong argument count"));
 
+  database db(app);
+
   packet_writer pw(output);
 
-  revision_id r_id(idx(args, 0)());
+  revision_id r_id(decode_hexenc(idx(args, 0)()));
   revision_data r_data;
 
-  N(app.db.revision_exists(r_id),
-    F("no such revision '%s'") % r_id);
-  app.db.get_revision(r_id, r_data);
-  pw.consume_revision_data(r_id,r_data);
+  N(db.revision_exists(r_id), F("no such revision '%s'") % r_id);
+  db.get_revision(r_id, r_data);
+  pw.consume_revision_data(r_id, r_data);
 }
 
 // Name: packets_for_certs
@@ -1410,16 +1449,19 @@ CMD_AUTOMATE(packets_for_certs, N_("REVID"),
   N(args.size() == 1,
     F("wrong argument count"));
 
+  database db(app);
+  project_t project(db);
   packet_writer pw(output);
 
-  revision_id r_id(idx(args, 0)());
+  revision_id r_id(decode_hexenc(idx(args, 0)()));
   vector< revision<cert> > certs;
 
-  N(app.db.revision_exists(r_id),
-    F("no such revision '%s'") % r_id);
-  app.get_project().get_revision_certs(r_id, certs);
-  for (size_t i = 0; i < certs.size(); ++i)
-    pw.consume_revision_cert(idx(certs,i));
+  N(db.revision_exists(r_id), F("no such revision '%s'") % r_id);
+  project.get_revision_certs(r_id, certs);
+
+  for (vector< revision<cert> >::const_iterator i = certs.begin();
+       i != certs.end(); i++)
+    pw.consume_revision_cert(*i);
 }
 
 // Name: packet_for_fdata
@@ -1440,15 +1482,16 @@ CMD_AUTOMATE(packet_for_fdata, N_("FILEID"),
   N(args.size() == 1,
     F("wrong argument count"));
 
+  database db(app);
+
   packet_writer pw(output);
 
-  file_id f_id(idx(args, 0)());
+  file_id f_id(decode_hexenc(idx(args, 0)()));
   file_data f_data;
 
-  N(app.db.file_version_exists(f_id),
-    F("no such file '%s'") % f_id);
-  app.db.get_file_version(f_id, f_data);
-  pw.consume_file_data(f_id,f_data);
+  N(db.file_version_exists(f_id), F("no such file '%s'") % f_id);
+  db.get_file_version(f_id, f_data);
+  pw.consume_file_data(f_id, f_data);
 }
 
 // Name: packet_for_fdelta
@@ -1470,18 +1513,20 @@ CMD_AUTOMATE(packet_for_fdelta, N_("OLD_FILE NEW_FILE"),
   N(args.size() == 2,
     F("wrong argument count"));
 
+  database db(app);
+
   packet_writer pw(output);
 
-  file_id f_old_id(idx(args, 0)());
-  file_id f_new_id(idx(args, 1)());
+  file_id f_old_id(decode_hexenc(idx(args, 0)()));
+  file_id f_new_id(decode_hexenc(idx(args, 1)()));
   file_data f_old_data, f_new_data;
 
-  N(app.db.file_version_exists(f_old_id),
+  N(db.file_version_exists(f_old_id),
     F("no such revision '%s'") % f_old_id);
-  N(app.db.file_version_exists(f_new_id),
+  N(db.file_version_exists(f_new_id),
     F("no such revision '%s'") % f_new_id);
-  app.db.get_file_version(f_old_id, f_old_data);
-  app.db.get_file_version(f_new_id, f_new_data);
+  db.get_file_version(f_old_id, f_old_data);
+  db.get_file_version(f_new_id, f_new_data);
   delta del;
   diff(f_old_data.inner(), f_new_data.inner(), del);
   pw.consume_file_delta(f_old_id, f_new_id, file_delta(del));
@@ -1508,12 +1553,14 @@ CMD_AUTOMATE(common_ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
   N(args.size() > 0,
     F("wrong argument count"));
 
+  database db(app);
+
   set<revision_id> ancestors, common_ancestors;
   vector<revision_id> frontier;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid((*i)());
-      N(app.db.revision_exists(rid), F("No such revision %s") % rid);
+      revision_id rid(decode_hexenc((*i)()));
+      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
       ancestors.clear();
       ancestors.insert(rid);
       frontier.push_back(rid);
@@ -1524,7 +1571,7 @@ CMD_AUTOMATE(common_ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
           if(!null_id(rid))
             {
               set<revision_id> parents;
-              app.db.get_revision_parents(rid, parents);
+              db.get_revision_parents(rid, parents);
               for (set<revision_id>::const_iterator i = parents.begin();
                    i != parents.end(); ++i)
                 {
@@ -1551,7 +1598,7 @@ CMD_AUTOMATE(common_ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
   for (set<revision_id>::const_iterator i = common_ancestors.begin();
        i != common_ancestors.end(); ++i)
     if (!null_id(*i))
-      output << (*i).inner()() << '\n';
+      output << *i << '\n';
 }
 
 // Name: branches
@@ -1574,16 +1621,16 @@ CMD_AUTOMATE(branches, "",
   N(args.size() == 0,
     F("no arguments needed"));
 
+  database db(app);
+  project_t project(db);
   set<branch_name> names;
 
-  app.get_project().get_branch_list(names, !app.opts.ignore_suspend_certs);
+  project.get_branch_list(names, !app.opts.ignore_suspend_certs);
 
   for (set<branch_name>::const_iterator i = names.begin();
        i != names.end(); ++i)
-    {
-      if (!app.lua.hook_ignore_branch(*i))
-        output << (*i) << '\n';
-    }
+    if (!app.lua.hook_ignore_branch(*i))
+      output << (*i) << '\n';
 }
 
 // Name: tags
@@ -1626,6 +1673,8 @@ CMD_AUTOMATE(tags, N_("[BRANCH_PATTERN]"),
   N(args.size() < 2,
     F("wrong argument count"));
 
+  database db(app);
+  project_t project(db);
   globish incl("*");
   bool filtering(false);
 
@@ -1640,13 +1689,13 @@ CMD_AUTOMATE(tags, N_("[BRANCH_PATTERN]"),
   prt.print_stanza(stz);
 
   set<tag_t> tags;
-  app.get_project().get_tags(tags);
+  project.get_tags(tags);
 
   for (set<tag_t>::const_iterator tag = tags.begin();
        tag != tags.end(); ++tag)
     {
       set<branch_name> branches;
-      app.get_project().get_revision_branches(tag->ident, branches);
+      project.get_revision_branches(tag->ident, branches);
 
       bool show(!filtering);
       vector<string> branch_names;
@@ -1654,6 +1703,8 @@ CMD_AUTOMATE(tags, N_("[BRANCH_PATTERN]"),
       for (set<branch_name>::const_iterator branch = branches.begin();
            branch != branches.end(); ++branch)
         {
+          // FIXME: again, hook_ignore_branch should probably be in the
+          //        database context...
           if (app.lua.hook_ignore_branch(*branch))
             continue;
 
@@ -1666,7 +1717,7 @@ CMD_AUTOMATE(tags, N_("[BRANCH_PATTERN]"),
         {
           basic_io::stanza stz;
           stz.push_str_pair(symbol("tag"), tag->name());
-          stz.push_hex_pair(symbol("revision"), tag->ident.inner());
+          stz.push_binary_pair(symbol("revision"), tag->ident.inner());
           stz.push_str_pair(symbol("signer"), tag->key());
           stz.push_str_multi(symbol("branches"), branch_names);
           prt.print_stanza(stz);
@@ -1721,38 +1772,28 @@ CMD_AUTOMATE(genkey, N_("KEYID PASSPHRASE"),
   N(args.size() == 2,
     F("wrong argument count"));
 
+  database db(app);
+  key_store keys(app);
+
   rsa_keypair_id ident;
   internalize_rsa_keypair_id(idx(args, 0), ident);
 
   utf8 passphrase = idx(args, 1);
 
-  bool exists = app.keys.key_pair_exists(ident);
-  if (app.db.database_specified())
-    {
-      transaction_guard guard(app.db);
-      exists = exists || app.db.public_key_exists(ident);
-      guard.commit();
-    }
-
-  N(!exists, F("key '%s' already exists") % ident);
-
-  keypair kp;
-  generate_key_pair(kp, passphrase);
-  app.keys.put_key_pair(ident, kp);
+  id pubhash, privhash;
+  keys.create_key_pair(db, ident, &passphrase, &pubhash, &privhash);
 
   basic_io::printer prt;
   basic_io::stanza stz;
-  hexenc<id> privhash, pubhash;
   vector<string> publocs, privlocs;
-  key_hash_code(ident, kp.pub, pubhash);
-  key_hash_code(ident, kp.priv, privhash);
-
+  if (db.database_specified())
+    publocs.push_back("database");
   publocs.push_back("keystore");
   privlocs.push_back("keystore");
 
   stz.push_str_pair(syms::name, ident());
-  stz.push_hex_pair(syms::public_hash, pubhash);
-  stz.push_hex_pair(syms::private_hash, privhash);
+  stz.push_binary_pair(syms::public_hash, pubhash);
+  stz.push_binary_pair(syms::private_hash, privhash);
   stz.push_str_multi(syms::public_location, publocs);
   stz.push_str_multi(syms::private_location, privlocs);
   prt.print_stanza(stz);
@@ -1780,28 +1821,8 @@ CMD_AUTOMATE(get_option, N_("OPTION"),
   N(args.size() == 1,
     F("wrong argument count"));
 
-  // this command requires a workspace to be run on
-  app.require_workspace();
-
-  system_path database_option;
-  branch_name branch_option;
-  rsa_keypair_id key_option;
-  system_path keydir_option;
-  app.work.get_ws_options(database_option, branch_option,
-                          key_option, keydir_option);
-
-  string opt = args[0]();
-
-  if (opt == "database")
-    output << database_option << '\n';
-  else if (opt == "branch")
-    output << branch_option << '\n';
-  else if (opt == "key")
-    output << key_option << '\n';
-  else if (opt == "keydir")
-    output << keydir_option << '\n';
-  else
-    N(false, F("'%s' is not a recognized workspace option") % opt);
+  workspace work(app);
+  work.print_ws_option(args[0], output);
 }
 
 // Name: get_content_changed
@@ -1833,18 +1854,21 @@ CMD_AUTOMATE(get_content_changed, N_("REV FILE"),
   N(args.size() == 2,
     F("wrong argument count"));
 
+  database db(app);
+
   roster_t new_roster;
   revision_id ident;
   marking_map mm;
 
-  ident = revision_id(idx(args, 0)());
-  N(app.db.revision_exists(ident),
+  ident = revision_id(decode_hexenc(idx(args, 0)()));
+  N(db.revision_exists(ident),
     F("no revision %s found in database") % ident);
-  app.db.get_roster(ident, new_roster, mm);
+  db.get_roster(ident, new_roster, mm);
 
   file_path path = file_path_external(idx(args,1));
   N(new_roster.has_node(path),
-    F("file %s is unknown for revision %s") % path % ident);
+    F("file %s is unknown for revision %s")
+    % path % ident);
 
   node_t node = new_roster.get_node(path);
   marking_map::const_iterator m = mm.find(node->self);
@@ -1856,7 +1880,7 @@ CMD_AUTOMATE(get_content_changed, N_("REV FILE"),
        i != mark.file_content.end(); ++i)
     {
       basic_io::stanza st;
-      st.push_hex_pair(basic_io::syms::content_mark, i->inner());
+      st.push_binary_pair(basic_io::syms::content_mark, i->inner());
       prt.print_stanza(st);
     }
     output.write(prt.buf.data(), prt.buf.size());
@@ -1895,18 +1919,20 @@ CMD_AUTOMATE(get_corresponding_path, N_("REV1 FILE REV2"),
   N(args.size() == 3,
     F("wrong argument count"));
 
+  database db(app);
+
   roster_t new_roster, old_roster;
   revision_id ident, old_ident;
 
-  ident = revision_id(idx(args, 0)());
-  N(app.db.revision_exists(ident),
+  ident = revision_id(decode_hexenc(idx(args, 0)()));
+  N(db.revision_exists(ident),
     F("no revision %s found in database") % ident);
-  app.db.get_roster(ident, new_roster);
+  db.get_roster(ident, new_roster);
 
-  old_ident = revision_id(idx(args, 2)());
-  N(app.db.revision_exists(old_ident),
+  old_ident = revision_id(decode_hexenc(idx(args, 2)()));
+  N(db.revision_exists(old_ident),
     F("no revision %s found in database") % old_ident);
-  app.db.get_roster(old_ident, old_roster);
+  db.get_roster(old_ident, old_roster);
 
   file_path path = file_path_external(idx(args,1));
   N(new_roster.has_node(path),
@@ -1945,33 +1971,35 @@ CMD_AUTOMATE(put_file, N_("[FILEID] CONTENTS"),
   N(args.size() == 1 || args.size() == 2,
     F("wrong argument count"));
 
+  database db(app);
+
   file_id sha1sum;
-  transaction_guard tr(app.db);
+  transaction_guard tr(db);
   if (args.size() == 1)
     {
       file_data dat(idx(args, 0)());
       calculate_ident(dat, sha1sum);
 
-      app.db.put_file(sha1sum, dat);
+      db.put_file(sha1sum, dat);
     }
   else if (args.size() == 2)
     {
       file_data dat(idx(args, 1)());
       calculate_ident(dat, sha1sum);
-      file_id base_id(idx(args, 0)());
-      N(app.db.file_version_exists(base_id),
+      file_id base_id(decode_hexenc(idx(args, 0)()));
+      N(db.file_version_exists(base_id),
         F("no file version %s found in database") % base_id);
 
       // put_file_version won't do anything if the target ID already exists,
       // but we can save the delta calculation by checking here too
-      if (!app.db.file_version_exists(sha1sum))
+      if (!db.file_version_exists(sha1sum))
         {
           file_data olddat;
-          app.db.get_file_version(base_id, olddat);
+          db.get_file_version(base_id, olddat);
           delta del;
           diff(olddat.inner(), dat.inner(), del);
 
-          app.db.put_file_version(base_id, sha1sum, file_delta(del));
+          db.put_file_version(base_id, sha1sum, file_delta(del));
         }
     }
   else I(false);
@@ -1998,6 +2026,8 @@ CMD_AUTOMATE(put_revision, N_("REVISION-DATA"),
   N(args.size() == 1,
     F("wrong argument count"));
 
+  database db(app);
+
   revision_t rev;
   read_revision(revision_data(idx(args, 0)()), rev);
 
@@ -2008,7 +2038,7 @@ CMD_AUTOMATE(put_revision, N_("REVISION-DATA"),
     {
       // calculate new manifest
       roster_t old_roster;
-      if (!null_id(e->first)) app.db.get_roster(e->first, old_roster);
+      if (!null_id(e->first)) db.get_roster(e->first, old_roster);
       roster_t new_roster = old_roster;
       editable_roster_base eros(new_roster, nis);
       e->second->apply_to(eros);
@@ -2029,7 +2059,7 @@ CMD_AUTOMATE(put_revision, N_("REVISION-DATA"),
 
   // If the database refuses the revision, make sure this is because it's
   // already there.
-  E(app.db.put_revision(id, rev) || app.db.revision_exists(id),
+  E(db.put_revision(id, rev) || db.revision_exists(id),
     F("missing prerequisite for revision %s") % id);
 
   output << id << '\n';
@@ -2055,17 +2085,17 @@ CMD_AUTOMATE(cert, N_("REVISION-ID NAME VALUE"),
   N(args.size() == 3,
     F("wrong argument count"));
 
-  cert c;
-  revision_id rid(idx(args, 0)());
+  database db(app);
+  key_store keys(app);
 
-  transaction_guard guard(app.db);
-  N(app.db.revision_exists(rid),
-    F("no such revision '%s'") % rid);
-  make_simple_cert(rid.inner(), cert_name(idx(args, 1)()),
-                   cert_value(idx(args, 2)()), app, c);
-  revision<cert> rc(c);
-  app.db.put_revision_cert(rc);
-  guard.commit();
+  hexenc<id> hrid(idx(args, 0)());
+  revision_id rid(decode_hexenc(hrid()));
+  N(db.revision_exists(rid),
+    F("no such revision '%s'") % hrid);
+
+  cache_user_key(app.opts, app.lua, db, keys);
+  put_simple_revision_cert(db, keys, rid, cert_name(idx(args, 1)()),
+                           cert_value(idx(args, 2)()));
 }
 
 // Name: get_db_variables
@@ -2088,6 +2118,7 @@ CMD_AUTOMATE(get_db_variables, N_("[DOMAIN]"),
   N(args.size() < 2,
     F("wrong argument count"));
 
+  database db(app);
   bool filter_by_domain = false;
   var_domain filter;
   if (args.size() == 1)
@@ -2097,7 +2128,7 @@ CMD_AUTOMATE(get_db_variables, N_("[DOMAIN]"),
     }
 
   map<var_key, var_value> vars;
-  app.db.get_vars(vars);
+  db.get_vars(vars);
 
   var_domain cur_domain;
   basic_io::stanza st;
@@ -2157,11 +2188,13 @@ CMD_AUTOMATE(set_db_variable, N_("DOMAIN NAME VALUE"),
   N(args.size() == 3,
     F("wrong argument count"));
 
+  database db(app);
+
   var_domain domain = var_domain(idx(args, 0)());
   utf8 name = idx(args, 1);
   utf8 value = idx(args, 2);
   var_key key(domain, var_name(name()));
-  app.db.set_var(key, var_value(value()));
+  db.set_var(key, var_value(value()));
 }
 
 // Name: drop_db_variables
@@ -2185,20 +2218,22 @@ CMD_AUTOMATE(drop_db_variables, N_("DOMAIN [NAME]"),
   N(args.size() == 1 || args.size() == 2,
     F("wrong argument count"));
 
+  database db(app);
+
   var_domain domain(idx(args, 0)());
 
   if (args.size() == 2)
     {
       var_name name(idx(args, 1)());
       var_key  key(domain, name);
-      N(app.db.var_exists(key),
+      N(db.var_exists(key),
         F("no var with name %s in domain %s") % name % domain);
-      app.db.clear_var(key);
+      db.clear_var(key);
     }
   else
     {
       map<var_key, var_value> vars;
-      app.db.get_vars(vars);
+      db.get_vars(vars);
       bool found_something = false;
 
       for (map<var_key, var_value>::const_iterator i = vars.begin();
@@ -2207,7 +2242,7 @@ CMD_AUTOMATE(drop_db_variables, N_("DOMAIN [NAME]"),
           if (i->first.first == domain)
             {
               found_something = true;
-              app.db.clear_var(i->first);
+              db.clear_var(i->first);
             }
         }
 
