@@ -1,38 +1,14 @@
 /*************************************************
 * KeyUsage Source File                           *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2007 The Botan Project                *
 *************************************************/
 
 #include <botan/asn1_obj.h>
-#include <botan/bit_ops.h>
+#include <botan/der_enc.h>
+#include <botan/ber_dec.h>
 
 namespace Botan {
 
-namespace DER {
-
-/*************************************************
-* DER encode a KeyUsage BIT STRING               *
-*************************************************/
-void encode(DER_Encoder& encoder, Key_Constraints usage)
-   {
-   if(usage == NO_CONSTRAINTS)
-      throw Encoding_Error("Cannot encode zero usage constraints");
-
-   const u32bit unused_bits = low_bit(usage) - 1;
-
-   SecureVector<byte> der;
-   der.append(BIT_STRING);
-   der.append(2 + ((unused_bits < 8) ? 1 : 0));
-   der.append(unused_bits % 8);
-   der.append((usage >> 8) & 0xFF);
-   if(usage & 0xFF)
-      der.append(usage & 0xFF);
-
-   encoder.add_raw_octets(der);
-   }
-
-}
-
 namespace BER {
 
 /*************************************************
@@ -54,7 +30,7 @@ void decode(BER_Decoder& source, Key_Constraints& key_usage)
    obj.value[obj.value.size()-1] &= mask;
 
    u16bit usage = 0;
-   for(u32bit j = 1; j != obj.value.size(); j++)
+   for(u32bit j = 1; j != obj.value.size(); ++j)
       usage = (obj.value[j] << 8) | usage;
 
    key_usage = Key_Constraints(usage);
