@@ -1,11 +1,10 @@
 /*************************************************
 * Base Classes Source File                       *
-* (C) 1999-2006 The Botan Project                *
+* (C) 1999-2007 The Botan Project                *
 *************************************************/
 
 #include <botan/base.h>
 #include <botan/version.h>
-#include <botan/parsing.h>
 #include <botan/util.h>
 #include <botan/config.h>
 
@@ -151,7 +150,7 @@ void BufferedComputation::update(const MemoryRegion<byte>& in)
 *************************************************/
 void BufferedComputation::update(const std::string& str)
    {
-   update((const byte*)str.c_str(), str.size());
+   update(reinterpret_cast<const byte*>(str.data()), str.size());
    }
 
 /*************************************************
@@ -243,21 +242,4 @@ u32bit RandomNumberGenerator::add_entropy(EntropySource& source,
    return entropy_estimate(buffer, bytes_gathered);
    }
 
-/*************************************************
-* Return the version as a string                 *
-*************************************************/
-std::string version_string()
-   {
-   return "Botan " + to_string(version_major()) + "." +
-                     to_string(version_minor()) + "." +
-                     to_string(version_patch());
-   }
-
-/*************************************************
-* Return parts of the version as integers        *
-*************************************************/
-u32bit version_major() { return BOTAN_VERSION_MAJOR; }
-u32bit version_minor() { return BOTAN_VERSION_MINOR; }
-u32bit version_patch() { return BOTAN_VERSION_PATCH; }
-
 }
