@@ -1,14 +1,12 @@
 /*************************************************
 * Botan Core Interface Header File               *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2007 The Botan Project                *
 *************************************************/
 
 #include <botan/base.h>
-#include <botan/conf.h>
+#include <botan/config.h>
 #include <botan/init.h>
 #include <botan/lookup.h>
 #include <botan/rng.h>
 #include <botan/version.h>
-#include <botan/bit_ops.h>
 #include <botan/parsing.h>
-
