@@ -1,10 +1,11 @@
 /*************************************************
 * DataSource Source File                         *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2007 The Botan Project                *
 *************************************************/
 
 #include <botan/data_src.h>
 #include <fstream>
+#include <algorithm>
 
 namespace Botan {
 
@@ -31,7 +32,7 @@ u32bit DataSource::discard_next(u32bit n)
    {
    u32bit discarded = 0;
    byte dummy;
-   for(u32bit j = 0; j != n; j++)
+   for(u32bit j = 0; j != n; ++j)
       discarded = read_byte(dummy);
    return discarded;
    }
@@ -87,18 +88,27 @@ DataSource_Memory::DataSource_Memory(const MemoryRegion<byte>& in)
    offset = 0;
    }
 
+/*************************************************
+* DataSource_Memory Constructor                  *
+*************************************************/
+DataSource_Memory::DataSource_Memory(const std::string& in)
+   {
+   source.set(reinterpret_cast<const byte*>(in.data()), in.length());
+   offset = 0;
+   }
+
 /*************************************************
 * Read from a stream                             *
 *************************************************/
 u32bit DataSource_Stream::read(byte out[], u32bit length)
    {
-   source->read((char*)out, length);
+   source->read(reinterpret_cast<char*>(out), length);
    if(source->bad())
       throw Stream_IO_Error("DataSource_Stream::read: Source failure");
 
    u32bit got = source->gcount();
    total_read += got;
-   return (u32bit)got;
+   return got;
    }
 
 /*************************************************
@@ -114,7 +124,7 @@ u32bit DataSource_Stream::peek(byte out[], u32bit length, u32bit offset) const
    if(offset)
       {
       SecureVector<byte> buf(offset);
-      source->read((char*)buf.begin(), buf.size());
+      source->read(reinterpret_cast<char*>(buf.begin()), buf.size());
       if(source->bad())
          throw Stream_IO_Error("DataSource_Stream::peek: Source failure");
       got = source->gcount();
@@ -122,7 +132,7 @@ u32bit DataSource_Stream::peek(byte out[], u32bit length, u32bit offset) const
 
    if(got == offset)
       {
-      source->read((char*)out, length);
+      source->read(reinterpret_cast<char*>(out), length);
       if(source->bad())
          throw Stream_IO_Error("DataSource_Stream::peek: Source failure");
       got = source->gcount();
