@@ -5,7 +5,7 @@
 
 #include <botan/kdf.h>
 #include <botan/lookup.h>
-#include <botan/bit_ops.h>
+#include <botan/loadstor.h>
 #include <algorithm>
 #include <memory>
 
