@@ -1,11 +1,13 @@
 /*************************************************
 * Mutex Header File                              *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2007 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_MUTEX_H__
 #define BOTAN_MUTEX_H__
 
+#include <botan/exceptn.h>
+
 namespace Botan {
 
 /*************************************************
@@ -16,27 +18,51 @@ class Mutex
    public:
       virtual void lock() = 0;
       virtual void unlock() = 0;
-      virtual Mutex* clone() const = 0;
       virtual ~Mutex() {}
    };
 
+/*************************************************
+* Mutex Factory                                  *
+*************************************************/
+class Mutex_Factory
+   {
+   public:
+      virtual Mutex* make() = 0;
+      virtual ~Mutex_Factory() {}
+   };
+
+/*************************************************
+* Default Mutex Factory                          *
+*************************************************/
+class Default_Mutex_Factory : public Mutex_Factory
+   {
+   public:
+      Mutex* make();
+   };
+
 /*************************************************
 * Mutex Holding Class                            *
 *************************************************/
 class Mutex_Holder
    {
    public:
-      Mutex_Holder(Mutex* m) : mux(m) { mux->lock(); }
-      ~Mutex_Holder() { mux->unlock(); }
+      Mutex_Holder(Mutex*);
+      ~Mutex_Holder();
    private:
       Mutex* mux;
    };
 
 /*************************************************
-* Get/set a mutex                                *
+* Named Mutex Holder                             *
 *************************************************/
-Mutex* get_mutex();
-void initialize_mutex(Mutex*&);
+class Named_Mutex_Holder
+   {
+   public:
+      Named_Mutex_Holder(const std::string&);
+      ~Named_Mutex_Holder();
+   private:
+      const std::string mutex_name;
+   };
 
 }
 
