@@ -1,6 +1,6 @@
 /*************************************************
 * X.509 Self-Signed Certificate Header File      *
-* (C) 1999-2006 The Botan Project                *
+* (C) 1999-2007 The Botan Project                *
 *************************************************/
 
 #ifndef BOTAN_X509_SELF_H__
@@ -26,7 +26,7 @@ class X509_Cert_Options
       std::string state;
       std::string serial_number;
 
-      std::string email, uri, dns, xmpp;
+      std::string email, uri, dns, ip, xmpp;
 
       std::string challenge;
 
