@@ -10,7 +10,11 @@
 #include "base.hh"
 #include "cmd.hh"
 #include "app_state.hh"
+#include "database.hh"
+#include "project.hh"
 #include "rcs_import.hh"
+#include "keys.hh"
+#include "key_store.hh"
 
 using std::vector;
 
@@ -25,9 +29,7 @@ CMD(rcs_import, "rcs_import", "", CMD_REF(debug), N_("RCSFILE..."),
 
   for (args_vector::const_iterator i = args.begin();
        i != args.end(); ++i)
-    {
-      test_parse_rcs_file(system_path((*i)()), app.db);
-    }
+    test_parse_rcs_file(system_path((*i)()));
 }
 
 
@@ -36,10 +38,27 @@ CMD(cvs_import, "cvs_import", "", CMD_REF(rcs), N_("CVSROOT"),
     "",
     options::opts::branch)
 {
+  database db(app);
+  key_store keys(app);
+  project_t project(db);
+
   if (args.size() != 1)
     throw usage(execid);
 
-  import_cvs_repo(system_path(idx(args, 0)()), app);
+  N(app.opts.branchname() != "",
+    F("need base --branch argument for importing"));
+
+  system_path cvsroot(idx(args, 0)());
+  require_path_is_directory(cvsroot,
+                            F("path %s does not exist") % cvsroot,
+                            F("'%s' is not a directory") % cvsroot);
+
+  // make sure we can sign certs using the selected key; also requests
+  // the password (if necessary) up front rather than after some arbitrary
+  // amount of work
+  cache_user_key(app.opts, app.lua, db, keys);
+
+  import_cvs_repo(project, keys, cvsroot, app.opts.branchname);
 }
 
 
