@@ -15,6 +15,7 @@
 #include "cset.hh"
 #include "sanity.hh"
 #include "safe_map.hh"
+#include "transforms.hh"
 
 using std::set;
 using std::map;
@@ -290,7 +291,7 @@ print_cset(basic_io::printer & printer,
     {
       basic_io::stanza st;
       st.push_file_pair(syms::add_file, i->first);
-      st.push_hex_pair(syms::content, i->second.inner());
+      st.push_binary_pair(syms::content, i->second.inner());
       printer.print_stanza(st);
     }
 
@@ -299,8 +300,8 @@ print_cset(basic_io::printer & printer,
     {
       basic_io::stanza st;
       st.push_file_pair(syms::patch, i->first);
-      st.push_hex_pair(syms::from, i->second.first.inner());
-      st.push_hex_pair(syms::to, i->second.second.inner());
+      st.push_binary_pair(syms::from, i->second.first.inner());
+      st.push_binary_pair(syms::to, i->second.second.inner());
       printer.print_stanza(st);
     }
 
@@ -393,7 +394,7 @@ parse_cset(basic_io::parser & parser,
       prev_path = p1;
       parser.esym(syms::content);
       parser.hex(t1);
-      safe_insert(cs.files_added, make_pair(p1, file_id(t1)));
+      safe_insert(cs.files_added, make_pair(p1, file_id(decode_hexenc(t1))));
     }
 
   prev_path.clear();
@@ -408,7 +409,8 @@ parse_cset(basic_io::parser & parser,
       parser.esym(syms::to);
       parser.hex(t2);
       safe_insert(cs.deltas_applied,
-                  make_pair(p1, make_pair(file_id(t1), file_id(t2))));
+                  make_pair(p1, make_pair(file_id(decode_hexenc(t1)),
+                                          file_id(decode_hexenc(t2)))));
     }
 
   prev_pair.first.clear();
@@ -469,6 +471,7 @@ dump(cset const & cs, string & out)
 }
 
 #ifdef BUILD_UNIT_TESTS
+#include "transforms.hh"
 #include "unit_tests.hh"
 
 #include "roster.hh"
@@ -687,9 +690,9 @@ UNIT_TEST(cset, cset_written)
   {
     L(FL("TEST: cset writing - normalisation"));
     cset cs; MM(cs);
-    file_id f1(string("1234567800000000000000000000000000000000"));
-    file_id f2(string("9876543212394657263900000000000000000000"));
-    file_id f3(string("0000000000011111111000000000000000000000"));
+    file_id f1(decode_hexenc("1234567800000000000000000000000000000000"));
+    file_id f2(decode_hexenc("9876543212394657263900000000000000000000"));
+    file_id f3(decode_hexenc("0000000000011111111000000000000000000000"));
 
     file_path foo = file_path_internal("foo");
     file_path foo_quux = file_path_internal("foo/quux");
@@ -759,8 +762,8 @@ UNIT_TEST(cset, basic_csets)
 
   editable_roster_base tree(r, nis);
 
-  file_id f1(string("0000000000000000000000000000000000000001"));
-  file_id f2(string("0000000000000000000000000000000000000002"));
+  file_id f1(decode_hexenc("0000000000000000000000000000000000000001"));
+  file_id f2(decode_hexenc("0000000000000000000000000000000000000002"));
 
   file_path root;
   file_path foo = file_path_internal("foo");
@@ -934,8 +937,8 @@ UNIT_TEST(cset, invalid_csets)
   MM(r);
   editable_roster_base tree(r, nis);
 
-  file_id f1(string("0000000000000000000000000000000000000001"));
-  file_id f2(string("0000000000000000000000000000000000000002"));
+  file_id f1(decode_hexenc("0000000000000000000000000000000000000001"));
+  file_id f2(decode_hexenc("0000000000000000000000000000000000000002"));
 
   file_path root;
   file_path foo = file_path_internal("foo");
@@ -1104,7 +1107,7 @@ UNIT_TEST(cset, root_dir)
   MM(r);
   editable_roster_base tree(r, nis);
 
-  file_id f1(string("0000000000000000000000000000000000000001"));
+  file_id f1(decode_hexenc("0000000000000000000000000000000000000001"));
 
   file_path root, baz = file_path_internal("baz");
 
