@@ -10,8 +10,9 @@
 #include "base.hh"
 #include "legacy.hh"
 #include "basic_io.hh"
-#include "app_state.hh"
 #include "constants.hh"
+#include "database.hh"
+#include "transforms.hh"
 
 using std::make_pair;
 using std::string;
@@ -81,7 +82,7 @@ namespace legacy
     string tmp;
     parser.esym(syms::old_revision);
     parser.hex(tmp);
-    old_rev = revision_id(tmp);
+    old_rev = revision_id(decode_hexenc(tmp));
     parser.esym(syms::old_manifest);
     parser.hex();
 
@@ -122,13 +123,13 @@ namespace legacy
 
   // cf. revision.cc:parse_revision in the pre-roster code.
   void
-  get_manifest_and_renames_for_rev(app_state & app,
+  get_manifest_and_renames_for_rev(database & db,
                                    revision_id const & ident,
                                    manifest_id & mid,
                                    renames_map & renames)
   {
     revision_data dat;
-    app.db.get_revision(ident, dat);
+    db.get_revision(ident, dat);
     basic_io::input_source src(dat.inner()(), "revision");
     basic_io::tokenizer tok(src);
     basic_io::parser pars(tok);
@@ -136,7 +137,7 @@ namespace legacy
     pars.esym(syms::new_manifest);
     string tmp;
     pars.hex(tmp);
-    mid = manifest_id(tmp);
+    mid = manifest_id(decode_hexenc(tmp));
     while (pars.symp(syms::old_revision))
       extract_renames(pars, renames);
   }
@@ -163,7 +164,7 @@ namespace legacy
         else
           file_name = dat().substr(file_name_begin, pos - file_name_begin);
         man.insert(make_pair(file_path_internal(file_name),
-                             hexenc<id>(ident)));
+                             file_id(decode_hexenc(ident))));
         // skip past the '\n'
         ++pos;
       }
