@@ -133,14 +133,6 @@ merkle_node::get_raw_slot(size_t slot, id & i) const
   i = idx(this->slots, slot);
 }
 
-void
-merkle_node::get_hex_slot(size_t slot, hexenc<id> & val) const
-{
-  id i;
-  get_raw_slot(slot, i);
-  encode_hexenc(i, val);
-}
-
 void
 merkle_node::set_raw_slot(size_t slot, id const & val)
 {
@@ -148,14 +140,6 @@ merkle_node::set_raw_slot(size_t slot, id const & val)
   idx(this->slots, slot) = val;
 }
 
-void
-merkle_node::set_hex_slot(size_t slot, hexenc<id> const & val)
-{
-  id i;
-  decode_hexenc(val, i);
-  set_raw_slot(slot, i);
-}
-
 void
 merkle_node::extended_prefix(size_t slot,
                              dynamic_bitset<unsigned char> & extended) const
