@@ -17,22 +17,19 @@
 #include <stack>
 #include <stdexcept>
 #include "vector.hh"
-
-#include <unistd.h>
+#include <cstring> // memset
 
 #include <boost/shared_ptr.hpp>
 #include <boost/scoped_ptr.hpp>
 #include "lexical_cast.hh"
 #include <boost/tokenizer.hpp>
 
-#include "app_state.hh"
 #include "cert.hh"
 #include "constants.hh"
 #include "cycle_detector.hh"
 #include "database.hh"
 #include "file_io.hh"
 #include "interner.hh"
-#include "keys.hh"
 #include "paths.hh"
 #include "platform-wrapped.hh"
 #include "project.hh"
@@ -42,6 +39,8 @@
 #include "sanity.hh"
 #include "transforms.hh"
 #include "ui.hh"
+#include "roster.hh"
+#include "xdelta.hh"
 
 using std::make_pair;
 using std::map;
@@ -451,10 +450,10 @@ construct_version(vector< piece > const & source_lines,
 // DB is stupid, but it's also stupid to put raw edge insert methods on the
 // DB itself. or is it? hmm.. encapsulation vs. usage guidance..
 void
-rcs_put_raw_file_edge(hexenc<id> const & old_id,
-                      hexenc<id> const & new_id,
-                      delta const & del,
-                      database & db)
+rcs_put_raw_file_edge(database & db,
+                      file_id const & old_id,
+                      file_id const & new_id,
+                      delta const & del)
 {
   if (old_id == new_id)
     {
@@ -462,7 +461,7 @@ rcs_put_raw_file_edge(hexenc<id> const & old_id,
       return;
     }
 
-  if (db.file_version_exists(file_id(old_id)))
+  if (db.file_version_exists(old_id))
     {
       // we already have a way to get to this old version,
       // no need to insert another reconstruction path
@@ -471,19 +470,18 @@ rcs_put_raw_file_edge(hexenc<id> const & old_id,
   else
     {
       I(db.file_or_manifest_base_exists(new_id, "files")
-        || db.delta_exists(new_id(), "file_deltas"));
-      db.put_file_delta(file_id(old_id), file_id(new_id), file_delta(del));
+        || db.delta_exists(new_id.inner(), "file_deltas"));
+      db.put_file_delta(old_id, new_id, file_delta(del));
     }
 }
 
 
 static void
-insert_into_db(data const & curr_data,
-               hexenc<id> const & curr_id,
+insert_into_db(database & db, data const & curr_data,
+               file_id const & curr_id,
                vector< piece > const & next_lines,
                data & next_data,
-               hexenc<id> & next_id,
-               database & db)
+               file_id & next_id)
 {
   // inserting into the DB
   // note: curr_lines is a "new" (base) version
@@ -496,8 +494,8 @@ insert_into_db(data const & curr_data,
   }
   delta del;
   diff(curr_data, next_data, del);
-  calculate_ident(next_data, next_id);
-  rcs_put_raw_file_edge(next_id, curr_id, del, db);
+  calculate_ident(file_data(next_data), next_id);
+  rcs_put_raw_file_edge(db, next_id, curr_id, del);
 }
 
 
@@ -560,12 +558,12 @@ lineage.
 
 
 static void
-process_branch(string const & begin_version,
+process_branch(database & db,
+               string const & begin_version,
                vector< piece > const & begin_lines,
                data const & begin_data,
-               hexenc<id> const & begin_id,
+               file_id const & begin_id,
                rcs_file const & r,
-               database & db,
                cvs_history & cvs)
 {
   string curr_version = begin_version;
@@ -574,7 +572,7 @@ process_branch(string const & begin_version,
                                            (begin_lines.begin(),
                                             begin_lines.end()));
   data curr_data(begin_data), next_data;
-  hexenc<id> curr_id(begin_id), next_id;
+  file_id curr_id(begin_id), next_id;
 
   while(! (r.deltas.find(curr_version) == r.deltas.end()))
     {
@@ -597,8 +595,8 @@ process_branch(string const & begin_version,
          L(FL("constructed RCS version %s, inserting into database") %
            next_version);
 
-         insert_into_db(curr_data, curr_id,
-                     *next_lines, next_data, next_id, db);
+         insert_into_db(db, curr_data, curr_id,
+                        *next_lines, next_data, next_id);
       }
 
       // mark the beginning-of-branch time and state of this file if
@@ -627,7 +625,7 @@ process_branch(string const & begin_version,
         {
           string branch;
           data branch_data;
-          hexenc<id> branch_id;
+          file_id branch_id;
           vector< piece > branch_lines;
           bool priv = false;
           map<string, string>::const_iterator be = cvs.branch_first_entries.find(*i);
@@ -640,11 +638,11 @@ process_branch(string const & begin_version,
           L(FL("following RCS branch %s = '%s'") % (*i) % branch);
 
           construct_version(*curr_lines, *i, branch_lines, r);
-          insert_into_db(curr_data, curr_id,
-                         branch_lines, branch_data, branch_id, db);
+          insert_into_db(db, curr_data, curr_id,
+                         branch_lines, branch_data, branch_id);
 
           cvs.push_branch(branch, priv);
-          process_branch(*i, branch_lines, branch_data, branch_id, r, db, cvs);
+          process_branch(db, *i, branch_lines, branch_data, branch_id, r, cvs);
           cvs.pop_branch();
 
           L(FL("finished RCS branch %s = '%s'") % (*i) % branch);
@@ -665,7 +663,8 @@ process_branch(string const & begin_version,
 
 
 static void
-import_rcs_file_with_cvs(string const & filename, database & db, cvs_history & cvs)
+import_rcs_file_with_cvs(database & db, string const & filename,
+                         cvs_history & cvs)
 {
   rcs_file r;
   L(FL("parsing RCS file %s") % filename);
@@ -677,14 +676,13 @@ import_rcs_file_with_cvs(string const & filename, database & db, cvs_history & c
     I(r.deltatexts.find(r.admin.head) != r.deltatexts.end());
     I(r.deltas.find(r.admin.head) != r.deltas.end());
 
-    hexenc<id> id;
-    data dat(r.deltatexts.find(r.admin.head)->second->text);
-    calculate_ident(dat, id);
-    file_id fid(id);
+    file_id fid;
+    file_data dat(r.deltatexts.find(r.admin.head)->second->text);
+    calculate_ident(dat, fid);
 
-    cvs.set_filename (filename, fid);
+    cvs.set_filename(filename, fid);
     cvs.index_branchpoint_symbols (r);
-    db.put_file(fid, file_data(dat));
+    db.put_file(fid, dat);
 
     {
       // create the head state in case it is a loner
@@ -696,16 +694,15 @@ import_rcs_file_with_cvs(string const & filename, database & db, cvs_history & c
 
     global_pieces.reset();
     global_pieces.index_deltatext(r.deltatexts.find(r.admin.head)->second, head_lines);
-    process_branch(r.admin.head, head_lines, dat, id, r, db, cvs);
+    process_branch(db, r.admin.head, head_lines, dat.inner(), fid, r, cvs);
     global_pieces.reset();
   }
 
   ui.set_tick_trailer("");
 }
 
-
 void
-test_parse_rcs_file(system_path const & filename, database & db)
+test_parse_rcs_file(system_path const & filename)
 {
   cvs_history cvs;
 
@@ -900,7 +897,7 @@ public:
       {
         try
           {
-            import_rcs_file_with_cvs(file, db, cvs);
+            import_rcs_file_with_cvs(db, file, cvs);
           }
         catch (oops const & o)
           {
@@ -1009,7 +1006,9 @@ struct
 cluster_consumer
 {
   cvs_history & cvs;
-  app_state & app;
+  key_store & keys;
+  project_t & project;
+
   string const & branchname;
   cvs_branch const & branch;
   set<file_path> created_dirs;
@@ -1036,8 +1035,9 @@ cluster_consumer
   editable_roster_base editable_ros;
   revision_id parent_rid, child_rid;
 
-  cluster_consumer(cvs_history & cvs,
-                   app_state & app,
+  cluster_consumer(project_t & project,
+                   key_store & keys,
+                   cvs_history & cvs,
                    string const & branchname,
                    cvs_branch const & branch,
                    ticker & n_revs);
@@ -1066,14 +1066,15 @@ typedef set<cluster_ptr, cluster_ptr_lt>
 cluster_set;
 
 void
-import_branch(cvs_history & cvs,
-              app_state & app,
+import_branch(project_t & project,
+              key_store & keys,
+              cvs_history & cvs,
               string const & branchname,
               shared_ptr<cvs_branch> const & branch,
               ticker & n_revs)
 {
   cluster_set clusters;
-  cluster_consumer cons(cvs, app, branchname, *branch, n_revs);
+  cluster_consumer cons(project, keys, cvs, branchname, *branch, n_revs);
   unsigned long commits_remaining = branch->lineage.size();
 
   // step 1: sort the lineage
@@ -1197,24 +1198,20 @@ import_branch(cvs_history & cvs,
 }
 
 void
-import_cvs_repo(system_path const & cvsroot,
-                app_state & app)
+import_cvs_repo(project_t & project,
+                key_store & keys,
+                system_path const & cvsroot,
+                branch_name const & branchname)
+
 {
   N(!directory_exists(cvsroot / "CVSROOT"),
     F("%s appears to be a CVS repository root directory\n"
       "try importing a module instead, with 'cvs_import %s/<module_name>")
     % cvsroot % cvsroot);
 
-  {
-    // early short-circuit to avoid failure after lots of work
-    rsa_keypair_id key;
-    get_user_key(key, app);
-    require_password(key, app);
-  }
-
   cvs_history cvs;
-  N(app.opts.branchname() != "", F("need base --branch argument for importing"));
-  cvs.base_branch = app.opts.branchname();
+
+  cvs.base_branch = branchname();
 
   // push the trunk
   cvs.trunk = shared_ptr<cvs_branch>(new cvs_branch());
@@ -1222,12 +1219,9 @@ import_cvs_repo(system_path const & cvsroot,
   cvs.bstk.push(cvs.branch_interner.intern(cvs.base_branch));
 
   {
-    transaction_guard guard(app.db);
-    cvs_tree_walker walker(cvs, app.db);
-    require_path_is_directory(cvsroot,
-                              F("path %s does not exist") % cvsroot,
-                              F("'%s' is not a directory") % cvsroot);
-    app.db.ensure_open();
+    transaction_guard guard(project.db);
+    cvs_tree_walker walker(cvs, project.db);
+    project.db.ensure_open();
     change_current_working_dir(cvsroot);
     walk_tree(file_path(), walker);
     guard.commit();
@@ -1239,12 +1233,12 @@ import_cvs_repo(system_path const & cvsroot,
 
   while (cvs.branches.size() > 0)
     {
-      transaction_guard guard(app.db);
+      transaction_guard guard(project.db);
       map<string, shared_ptr<cvs_branch> >::const_iterator i = cvs.branches.begin();
       string branchname = i->first;
       shared_ptr<cvs_branch> branch = i->second;
       L(FL("branch %s has %d entries") % branchname % branch->lineage.size());
-      import_branch(cvs, app, branchname, branch, n_revs);
+      import_branch(project, keys, cvs, branchname, branch, n_revs);
 
       // free up some memory
       cvs.branches.erase(branchname);
@@ -1252,38 +1246,37 @@ import_cvs_repo(system_path const & cvsroot,
     }
 
   {
-    transaction_guard guard(app.db);
+    transaction_guard guard(project.db);
     L(FL("trunk has %d entries") % cvs.trunk->lineage.size());
-    import_branch(cvs, app, cvs.base_branch, cvs.trunk, n_revs);
+    import_branch(project, keys, cvs, cvs.base_branch, cvs.trunk, n_revs);
     guard.commit();
   }
 
   // now we have a "last" rev for each tag
   {
     ticker n_tags(_("tags"), "t", 1);
-    transaction_guard guard(app.db);
+    transaction_guard guard(project.db);
     for (map<unsigned long, pair<time_t, revision_id> >::const_iterator i = cvs.resolved_tags.begin();
          i != cvs.resolved_tags.end(); ++i)
       {
         string tag = cvs.tag_interner.lookup(i->first);
         ui.set_tick_trailer("marking tag " + tag);
-        app.get_project().put_tag(i->second.second, tag);
+        project.put_tag(keys, i->second.second, tag);
         ++n_tags;
       }
     guard.commit();
   }
-
-
-  return;
 }
 
-cluster_consumer::cluster_consumer(cvs_history & cvs,
-                                   app_state & app,
+cluster_consumer::cluster_consumer(project_t & project,
+                                   key_store & keys,
+                                   cvs_history & cvs,
                                    string const & branchname,
                                    cvs_branch const & branch,
                                    ticker & n_revs)
   : cvs(cvs),
-    app(app),
+    keys(keys),
+    project(project),
     branchname(branchname),
     branch(branch),
     n_revisions(n_revs),
@@ -1341,7 +1334,7 @@ cluster_consumer::store_revisions()
 {
   for (vector<prepared_revision>::const_iterator i = preps.begin();
        i != preps.end(); ++i)
-    if (app.db.put_revision(i->rid, *(i->rev)))
+    if (project.db.put_revision(i->rid, *(i->rev)))
       {
         store_auxiliary_certs(*i);
         ++n_revisions;
@@ -1372,11 +1365,11 @@ cluster_consumer::store_auxiliary_certs(prepared_revision const & p)
         }
     }
 
-  app.get_project().put_standard_certs(p.rid,
-                                       branch_name(branchname),
-                                       utf8(cvs.changelog_interner.lookup(p.changelog)),
-                                       date_t::from_unix_epoch(p.time),
-                                       utf8(cvs.author_interner.lookup(p.author)));
+  project.put_standard_certs(keys, p.rid,
+                             branch_name(branchname),
+                             utf8(cvs.changelog_interner.lookup(p.changelog)),
+                             date_t::from_unix_epoch(p.time),
+                             cvs.author_interner.lookup(p.author));
 }
 
 void
@@ -1408,7 +1401,8 @@ cluster_consumer::build_cset(cvs_cluster const & c,
           if (e == live_files.end())
             {
               add_missing_parents(pth.dirname(), cs);
-              L(FL("adding entry state '%s' on '%s'") % fid % pth);
+              L(FL("adding entry state '%s' on '%s'")
+                % fid % pth);
               safe_insert(cs.files_added, make_pair(pth, fid));
               live_files[i->first] = i->second.version;
             }
@@ -1416,7 +1410,9 @@ cluster_consumer::build_cset(cvs_cluster const & c,
             {
               file_id old_fid(cvs.file_version_interner.lookup(e->second));
               L(FL("applying state delta on '%s' : '%s' -> '%s'")
-                % pth % old_fid % fid);
+                % pth
+                % old_fid
+                % fid);
               safe_insert(cs.deltas_applied,
                           make_pair(pth, make_pair(old_fid, fid)));
               live_files[i->first] = i->second.version;
@@ -1427,7 +1423,8 @@ cluster_consumer::build_cset(cvs_cluster const & c,
           map<cvs_path, cvs_version>::const_iterator e = live_files.find(i->first);
           if (e != live_files.end())
             {
-              L(FL("deleting entry state '%s' on '%s'") % fid % pth);
+              L(FL("deleting entry state '%s' on '%s'")
+                % fid % pth);
               safe_insert(cs.nodes_deleted, pth);
               live_files.erase(i->first);
             }
