@@ -17,7 +17,7 @@ revs[1] = base_revision()
 
 netsync.pull("testbranch")
 
-for _,x in revs do
+for _,x in pairs(revs) do
   check_same_stdout(mtn("automate", "get_revision", x),
                     mtn2("automate", "get_revision", x))
   check_same_stdout(mtn("ls", "certs", x),
