@@ -20,7 +20,7 @@ end
 check(mtn("add", "file1"), 0, false, false)
 commit("test", "add file1")
 revs[1] = base_revision()
-check(raw_mtn("db", "execute", "select id from rosters"), 0, true, false)
+check(raw_mtn("db", "execute", "select hex(id) from rosters"), 0, true, false)
 check(tail("stdout", 1), 0, true)
 rosters[1] = trim(readfile("stdout"))
 
@@ -40,7 +40,7 @@ check(qgrep('database is good', "stderr"))
 -- remove file2 from the database invalidating roster2 and roster3
 -- both of which include this file
 
-dbex("delete from files where id='%s'", files[2])
+dbex("delete from files where id=x'%s'", files[2])
 
 check(mtn("db", "check", "--ticker=dot"), 1, false, true)
 check(not qgrep('database is good', "stderr"))
@@ -61,11 +61,11 @@ check(mtn("add", "fileY"), 0, false, false)
 commit("test", "to be removed")
 revs[4] = base_revision()
 copy("_MTN/revision", "saved_revision")
-dbex("delete from revisions where id='%s'", revs[4])
+dbex("delete from revisions where id=x'%s'", revs[4])
 -- revert to the old workspace state
 copy("saved_revision", "_MTN/revision")
 -- remove another file too
-dbex("delete from files where id='%s'", files[3])
+dbex("delete from files where id=x'%s'", files[3])
 
 check(mtn("db", "check", "--ticker=dot"), 1, false, true)
 check(qgrep('2 unreferenced file', "stderr"))
@@ -82,18 +82,18 @@ check(qgrep('2 mismatched child', "stderr"))
 xdelta_cc = "877cfe29db0f60dec63439857fe78673b9d55346"
 xdelta_hh = "68d15dc01398c7bb375b1a90fbb420bebef1bac7"
 
-dbex("insert into revision_ancestry values('%s', '%s')", xdelta_cc, xdelta_hh)
+dbex("insert into revision_ancestry values(x'%s', x'%s')", xdelta_cc, xdelta_hh)
 check(mtn("db", "check", "--ticker=dot"), 1, false, true)
 check(qgrep('3 mismatched parent', "stderr"))
 check(qgrep('3 mismatched child', "stderr"))
 check(qgrep('3 missing revision', "stderr"))
 
-dbex("delete from roster_deltas where id='%s'", rosters[1])
+dbex("delete from roster_deltas where id=x'%s'", rosters[1])
 check(mtn("db", "check", "--ticker=dot"), 1, false, true)
 -- ROSTER TODO: need check_sane_history equivalent in db check
 --check(grep '3 revisions with bad history' stderr, 0, false, false)
 
-dbex("delete from revisions where id='%s'", revs[1])
+dbex("delete from revisions where id=x'%s'", revs[1])
 check(mtn("db", "check", "--ticker=dot"), 1, false, true)
 check(qgrep('4 missing revision', "stderr"))
 -- ROSTER TODO
@@ -102,7 +102,7 @@ check(qgrep('4 missing revision', "stderr"))
 writefile("tosum", revs[2]..":comment:this is a test:tester@test.net:bogus sig")
 hash = sha1("tosum")
 
-dbex("insert into revision_certs values ('%s', '%s', 'comment', 'this is a test', 'tester@test.net', 'bogus sig')", hash, revs[2])
+dbex("insert into revision_certs values (x'%s', x'%s', 'comment', 'this is a test', 'tester@test.net', 'bogus sig')", hash, revs[2])
 check(mtn("db", "check", "--ticker=dot"), 1, false, true)
 check(qgrep('1 bad sig', "stderr"))
 
