@@ -1,4 +1,7 @@
 -- Test that --confdir gets passed properly on file: sync
+
+skip_if(ostype == "Windows") -- file: not supported on native Win32
+
 mtn_setup()
 
 copy("test.db", "test2.db")
@@ -18,8 +21,12 @@ end
 check(mtn("sync", "file:test2.db", "testbranch"), 0, true, false)
 
 n = 0
+
+
+testroot_unix = string.gsub(test.root, '\\', '/')
+
 for line in io.lines("checkfile") do
-   check(line == test.root)
+   check(line == testroot_unix)
    n = n + 1
 end
 check(n == 2)
