@@ -3,8 +3,10 @@
 // licensed to the public under the terms of the GNU GPL (>= 2)
 // see the file COPYING for details
 
+#include "base.hh"
 #include <sys/utsname.h>
 #include "sanity.hh"
+#include <ostream> // for operator<<
 
 void get_system_flavour(std::string & ident)
 {
