@@ -13,50 +13,33 @@
 #include "hash_map.hh"
 #include "sanity.hh"
 #include "vocab.hh"
+#include "char_classifiers.hh"
+#include "transforms.hh"
 
 using std::string;
 
 // verifiers for various types of data
 
-// the verify() stuff gets a little complicated; there doesn't seem to be a
-// really nice way to achieve what we want with c++'s type system.  the
-// problem is this: we want to give verify(file_path) and verify(local_path)
-// access to the internals of file_path and local_path, i.e. make them
-// friends, so they can normalize the file paths they're given.  this means
-// that verify() needs to be declared publically, so that the definition of
-// these classes can refer to them.  it also means that they -- and all other
-// ATOMIC types -- cannot fall back on a templated version of verify if no
-// other version is defined, because, well, the friend thing and the template
-// thing just don't work out, as far as I can tell.  So, every ATOMIC type
-// needs an explicitly defined verify() function, so we have both ATOMIC() and
-// ATOMIC_NOVERIFY() macros, the latter of which defines a type-specific noop
-// verify function.  DECORATE and ENCODING, on the other hand, cannot make use
-// of a trick like these, because they are template types themselves, and we
-// want to be able to define verify(hexenc<id>) without defining
-// verify(hexenc<data>) at the same time, for instance.  Fortunately, these
-// types never need to be friends with their verify functions (yet...), so we
-// _can_ use a templated fallback function.  This templated function is used
-// _only_ by DECORATE and ENCODING; it would be nice to make it take an
-// argument of type T1<T2> to document that, but for some reason that doesn't
-// work either.
-template <typename T>
-static inline void
-verify(T & val)
-{}
+// Every ENCODING and ATOMIC type not defined with the _NOVERIFY variant in
+// vocab_terms.hh must have a verify function defined here.  DECORATE types
+// use the verify function of their inner type.
 
-template <typename T>
-static inline void
-verify_full(T & val)
-{ val.ok = true; }
-
-// NOTE: _not_ verify_full; you use verify_full for ATOMICs, verify() for
-// everything else.
+// ENCODING types ... hexenc<id> has a fixed size, hexenc<other> doesn't.
+template <typename INNER>
 inline void
-verify(hexenc<id> & val)
+verify(hexenc<INNER> const & val)
 {
-  if (val.ok)
-    return;
+  for (string::const_iterator i = val().begin(); i != val().end(); ++i)
+    {
+      N(is_xdigit(*i),
+        F("bad character '%c' in '%s'") % *i % val);
+    }
+}
 
+template <>
+inline void
+verify(hexenc<id> const & val)
+{
   if (val().empty())
     return;
 
@@ -67,43 +50,51 @@ verify(hexenc<id> & val)
       N(is_xdigit(*i),
         F("bad character '%c' in id name '%s'") % *i % val);
     }
-  val.ok = true;
+}
+
+// ATOMIC types ...
+inline void
+verify(id & val)
+{
+  if (val().empty())
+    return;
+
+  N(val().size() == constants::idlen_bytes,
+    F("invalid ID '%s'") % val);
 }
 
 inline void
-verify_full(symbol & val)
+verify(symbol const & val)
 {
   for (string::const_iterator i = val().begin(); i != val().end(); ++i)
     {
       N(is_alnum(*i) || *i == '_',
         F("bad character '%c' in symbol '%s'") % *i % val);
     }
-
-  val.ok = true;
 }
 
 inline void
-verify_full(cert_name & val)
+verify(cert_name const & val)
 {
   string::size_type pos = val().find_first_not_of(constants::legal_cert_name_bytes);
   N(pos == string::npos,
     F("bad character '%c' in cert name '%s'") % val().at(pos) % val);
-
-  val.ok = true;
 }
 
 inline void
-verify_full(rsa_keypair_id & val)
+verify(rsa_keypair_id const & val)
 {
   string::size_type pos = val().find_first_not_of(constants::legal_key_name_bytes);
   N(pos == string::npos,
     F("bad character '%c' in key name '%s'") % val().at(pos) % val);
-
-  val.ok = true;
 }
 
+// These two may modify their argument, to set a more sensible value when
+// initializing from the empty string or the default constructor; therefore
+// they cannot take a const argument and must be friends with their class.
+
 inline void
-verify_full(netsync_session_key & val)
+verify(netsync_session_key & val)
 {
   if (val().size() == 0)
     {
@@ -113,12 +104,10 @@ verify_full(netsync_session_key & val)
 
   N(val().size() == constants::netsync_session_key_length_in_bytes,
     F("Invalid key length of %d bytes") % val().length());
-
-  val.ok = true;
 }
 
 inline void
-verify_full(netsync_hmac_value & val)
+verify(netsync_hmac_value & val)
 {
   if (val().size() == 0)
     {
@@ -128,8 +117,6 @@ verify_full(netsync_hmac_value & val)
 
   N(val().size() == constants::netsync_hmac_value_length_in_bytes,
     F("Invalid hmac length of %d bytes") % val().length());
-
-  val.ok = true;
 }
 
 
@@ -158,13 +145,14 @@ symtab_impl
 
 // Sometimes it's handy to have a non-colliding, meaningless id.
 
-hexenc<id>
+id
 fake_id()
 {
   static u32 counter = 0;
   ++counter;
   I(counter >= 1); // detect overflow
-  return hexenc<id>((FL("00000000000000000000000000000000%08x") % counter).str());
+  string s((FL("00000000000000000000000000000000%08x") % counter).str());
+  return id(decode_hexenc(s));
 }
 
 // instantiation of various vocab functions
@@ -173,45 +161,60 @@ fake_id()
 
 #include "vocab_macros.hh"
 #define ENCODING(enc) cc_ENCODING(enc)
+#define ENCODING_NOVERIFY(enc) cc_ENCODING_NOVERIFY(enc)
 #define DECORATE(dec) cc_DECORATE(dec)
 #define ATOMIC(ty) cc_ATOMIC(ty)
+#define ATOMIC_HOOKED(ty,hook) cc_ATOMIC(ty)
 #define ATOMIC_NOVERIFY(ty) cc_ATOMIC_NOVERIFY(ty)
+#define ATOMIC_BINARY(ty) cc_ATOMIC_BINARY(ty)
 
-#ifdef EXTERN
 #undef EXTERN
-#endif
 #define EXTERN
 
 #include "vocab_terms.hh"
 
 #undef EXTERN
 #undef ATOMIC
+#undef ATOMIC_HOOKED
+#undef ATOMIC_NOVERIFY
 #undef DECORATE
+#undef ENCODING
+#undef ENCODING_NOVERIFY
 
+template void dump(revision_id const & r, string &);
+template void dump(manifest_id const & r, string &);
+template void dump(file_id const & r, string &);
+template void dump(hexenc<id> const & r, string &);
+template void dump(rsa_pub_key const&, string &);
+template void dump(roster_data const & d, string &);
+template void dump(roster_delta const & d, string &);
+template void dump(manifest_data const & d, string &);
+template void dump(revision_data const & d, string &);
 
-template
-void dump<rsa_pub_key>(base64<rsa_pub_key> const&, string &);
+template std::ostream & operator<< <>(std::ostream &,    epoch<id> const &);
+template std::ostream & operator<< <>(std::ostream &,     file<id> const &);
+template std::ostream & operator<< <>(std::ostream &,   hexenc<id> const &);
+template std::ostream & operator<< <>(std::ostream &,      key<id> const &);
+template std::ostream & operator<< <>(std::ostream &, manifest<id> const &);
+template std::ostream & operator<< <>(std::ostream &, revision<id> const &);
+template std::ostream & operator<< <>(std::ostream &,   roster<id> const &);
 
-template
-void dump(revision_id const & r, string &);
+template std::ostream & operator<< <>(std::ostream &,    epoch<data> const &);
+template std::ostream & operator<< <>(std::ostream &,     file<data> const &);
+template std::ostream & operator<< <>(std::ostream &, manifest<data> const &);
+template std::ostream & operator<< <>(std::ostream &, revision<data> const &);
+template std::ostream & operator<< <>(std::ostream &,   roster<data> const &);
 
-template
-void dump(manifest_id const & r, string &);
-
-template
-void dump(file_id const & r, string &);
-
-template
-void dump(hexenc<id> const & r, string &);
-
-template
-void dump(roster_data const & d, string &);
-
-template
-void dump(roster_delta const & d, string &);
-
-template
-void dump(manifest_data const & d, string &);
+/*
+ * specializations for id, which allows the encoded id
+ * to be dumped out as a human readable, hex encoded
+ * string.
+ */
+template <>
+void dump (id const & obj, std::string & out)
+{
+  out = encode_hexenc(obj());
+}
 
 #ifdef BUILD_UNIT_TESTS
 
