@@ -10,10 +10,12 @@
 #include "base.hh"
 #include "sanity.hh"
 #include "ui.hh"
+#include "cset.hh"
 #include "simplestring_xform.hh"
 #include "revision.hh"
 #include "file_io.hh"
 #include "work.hh"
+#include "transforms.hh"
 
 #include "lexical_cast.hh"
 #include <exception>
@@ -112,6 +114,9 @@ workspace::write_ws_format()
 void
 workspace::check_ws_format()
 {
+  if (!workspace::found)
+    return;
+
   unsigned int format = get_ws_format();
 
   // Don't give user false expectations about format 0.
@@ -191,7 +196,7 @@ migrate_1_to_2()
       E(false, F("workspace is corrupt: reading %s: %s")
         % rev_path % e.what());
     }
-  revision_id base_rid(remove_ws(base_rev_data())); 
+  revision_id base_rid(decode_hexenc(remove_ws(base_rev_data())));
   MM(base_rid);
 
   cset workcs; 
